/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

#ifdef ENABLE_XINE

#include "zalbum.h"
#include "gtkxine.h"
#include "movieview.h"

#include "pixmaps/play.xpm"
#include "pixmaps/pause.xpm"
#include "pixmaps/stop.xpm"
#include "pixmaps/ff.xpm"

#include "pixmaps/prev.xpm"
#include "pixmaps/next.xpm"

MovieView *movieview = NULL;

static void cb_movieview_pause (GtkWidget * widget, MovieView * mv);
static void cb_movieview_play (GtkWidget * widget, MovieView * mv);
static void cb_movieview_stop (GtkWidget * widget, MovieView * mv);
static void cb_movieview_forward (GtkWidget * widget, MovieView * mv);
static void cb_movieview_prev (GtkWidget * widget, MovieView * mv);
static void cb_movieview_next (GtkWidget * widget, MovieView * mv);

static gint cb_movieview_key_press (GtkWidget * widget, GdkEventKey * event,
				    MovieView * mv);
static gint cb_movieview_button_press (GtkWidget * widget,
				       GdkEventButton * event,
				       MovieView * mv);

static gboolean cb_seek_released (GtkWidget * widget, GdkEventButton * event,
				  MovieView * mv);
static gboolean cb_seek_pressed (GtkWidget * widget, GdkEventButton * event,
				 MovieView * mv);
static int cb_update_seek (gpointer data);

static void free_rgb_buffer (guchar * pixels, gpointer data);

void
movieview_create (void)
{
    GtkWidget *iconw;

    movieview = g_new0 (MovieView, 1);
    movieview->is_fullscreen = FALSE;
    movieview->is_hide = TRUE;
    movieview->seek_lock = FALSE;

    movieview->container = gtk_frame_new (NULL);
    gtk_widget_ref (movieview->container);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "frame",
			      movieview->container,
			      (GtkDestroyNotify) gtk_widget_unref);

    gtk_widget_show (movieview->container);

    movieview->vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (movieview->vbox);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "vbox",
			      movieview->vbox,
			      (GtkDestroyNotify) gtk_widget_unref);

    gtk_widget_show (movieview->vbox);
    gtk_container_add (GTK_CONTAINER (movieview->container), movieview->vbox);

    movieview->event_box = gtk_event_box_new ();
    gtk_box_pack_start (GTK_BOX (movieview->vbox), movieview->event_box,
			TRUE, TRUE, 0);

    gtk_widget_show (movieview->event_box);

    movieview->xine = gtk_xine_new ();
    gtk_container_add (GTK_CONTAINER (movieview->event_box), movieview->xine);
    gtk_widget_show (movieview->xine);

    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (movieview->event_box), GTK_CAN_FOCUS);

    gtk_widget_add_events (GTK_WIDGET (movieview->event_box),
			   GDK_FOCUS_CHANGE
			   | GDK_BUTTON_PRESS_MASK | GDK_2BUTTON_PRESS
			   | GDK_KEY_PRESS | GDK_KEY_RELEASE
			   | GDK_BUTTON_RELEASE_MASK
			   | GDK_POINTER_MOTION_MASK
			   | GDK_POINTER_MOTION_HINT_MASK);


    gtk_signal_connect (GTK_OBJECT (movieview->event_box),
			"key_press_event",
			GTK_SIGNAL_FUNC (cb_movieview_key_press), movieview);

    gtk_signal_connect_after (GTK_OBJECT (movieview->event_box),
			      "button_press_event",
			      GTK_SIGNAL_FUNC (cb_movieview_button_press),
			      movieview);

    movieview->hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_ref (movieview->hbox);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "hbox1",
			      movieview->hbox,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->hbox);
    gtk_box_pack_start (GTK_BOX (movieview->vbox), movieview->hbox, FALSE,
			FALSE, 10);

#ifdef USE_GTK2
    movieview->toolbar = gtk_toolbar_new ();
#else
    movieview->toolbar =
	gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
#endif

    gtk_widget_ref (movieview->toolbar);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "toolbar1",
			      movieview->toolbar,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->toolbar);
    gtk_box_pack_start (GTK_BOX (movieview->hbox), movieview->toolbar,
			FALSE, FALSE, 10);

    iconw = pixbuf_create_pixmap_from_xpm_data (play_xpm);
    movieview->play_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (movieview->toolbar), NULL,
				 "play", "play", iconw, GTK_SIGNAL_FUNC
				 (cb_movieview_play), movieview);
    gtk_widget_ref (movieview->play_btn);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "button3",
			      movieview->play_btn,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->play_btn);

    iconw = pixbuf_create_pixmap_from_xpm_data (pause_xpm);
    movieview->pause_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (movieview->toolbar), NULL,
				 "pause", "pause", iconw, GTK_SIGNAL_FUNC
				 (cb_movieview_pause), movieview);
    gtk_widget_ref (movieview->pause_btn);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "button4",
			      movieview->pause_btn,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->pause_btn);

    iconw = pixbuf_create_pixmap_from_xpm_data (stop_xpm);
    movieview->stop_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (movieview->toolbar), NULL,
				 "stop", "stop", iconw, GTK_SIGNAL_FUNC
				 (cb_movieview_stop), movieview);
    gtk_widget_ref (movieview->stop_btn);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "button5",
			      movieview->stop_btn,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->stop_btn);

    iconw = pixbuf_create_pixmap_from_xpm_data (ff_xpm);
    movieview->forward_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (movieview->toolbar), NULL,
				 "forward", "forward", iconw,
				 GTK_SIGNAL_FUNC (cb_movieview_forward),
				 movieview);
    gtk_widget_ref (movieview->forward_btn);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "button7",
			      movieview->forward_btn,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->forward_btn);

    iconw = pixbuf_create_pixmap_from_xpm_data (prev_xpm);
    movieview->previous_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (movieview->toolbar), NULL,
				 "previous", "previous", iconw,
				 GTK_SIGNAL_FUNC (cb_movieview_prev),
				 movieview);
    gtk_widget_ref (movieview->previous_btn);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "button8",
			      movieview->previous_btn,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->previous_btn);

    iconw = pixbuf_create_pixmap_from_xpm_data (next_xpm);
    movieview->next_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (movieview->toolbar), NULL,
				 "next", "next", iconw, GTK_SIGNAL_FUNC
				 (cb_movieview_next), movieview);
    gtk_widget_ref (movieview->next_btn);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "button9",
			      movieview->next_btn,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->next_btn);

    movieview->seek_adj =
	gtk_adjustment_new (0.0, 0.0, 65536.0, 1.0, 10.0, 1.0);

    GTK_ADJUSTMENT (movieview->seek_adj)->value = 0;

    movieview->seek = gtk_hscale_new (GTK_ADJUSTMENT (movieview->seek_adj));
    gtk_widget_ref (movieview->seek);
    gtk_object_set_data_full (GTK_OBJECT (browser->window), "hscale1",
			      movieview->seek,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (movieview->seek);
    gtk_box_pack_start (GTK_BOX (movieview->hbox), movieview->seek, TRUE,
			TRUE, 10);
    gtk_scale_set_draw_value (GTK_SCALE (movieview->seek), FALSE);
    gtk_range_set_update_policy (GTK_RANGE (movieview->seek),
				 GTK_UPDATE_CONTINUOUS);

    gtk_signal_connect (GTK_OBJECT (movieview->seek),
			"button_press_event",
			GTK_SIGNAL_FUNC (cb_seek_pressed), movieview);
    gtk_signal_connect (GTK_OBJECT (movieview->seek),
			"button_release_event",
			GTK_SIGNAL_FUNC (cb_seek_released), movieview);

    movieview->seek_timer_id =
	gtk_timeout_add (500, cb_update_seek, movieview);
}

void
movieview_destroy (void)
{
    if (movieview)
    {
	if (!GTK_IS_XINE (movieview->xine))
	{
	    if (gtk_xine_is_playing (GTK_XINE (movieview->xine)))
		gtk_xine_stop (GTK_XINE (movieview->xine));
	}

	if (movieview->seek_timer_id)
	    gtk_timeout_remove (movieview->seek_timer_id);

	gtk_widget_destroy (movieview->xine);

	g_free (movieview);

	movieview = NULL;
    }
}

void
movieview_play (gchar * name)
{
    gtk_xine_play (GTK_XINE (movieview->xine), name, 0, 0);
    gtk_adjustment_set_value (GTK_ADJUSTMENT (movieview->seek_adj), 0);

    movieview->seek_timer_id =
	gtk_timeout_add (500, cb_update_seek, movieview);
}

void
movieview_stop (void)
{
    if (movieview)
    {
	if (movieview->is_hide)
	    return;

	if (movieview->seek_timer_id)
	    gtk_timeout_remove (movieview->seek_timer_id);

	if (gtk_xine_is_playing (GTK_XINE (movieview->xine)))
	    gtk_xine_stop (GTK_XINE (movieview->xine));
    }
}

void
movieview_fullscreen_mode_set (void)
{
    if (gtk_xine_is_playing (GTK_XINE (movieview->xine)))
    {
	gdk_keyboard_grab (movieview->event_box->window, TRUE,
			   GDK_CURRENT_TIME);
	gtk_grab_add (movieview->event_box);

	gtk_widget_grab_focus (GTK_WIDGET (movieview->event_box));

	gtk_xine_set_fullscreen (GTK_XINE (movieview->xine), TRUE);
	movieview->is_fullscreen = TRUE;
    }
}

void
movieview_create_thumb (void)
{
    if (gtk_xine_is_playing (GTK_XINE (movieview->xine)))
    {
	guchar *pixels;
	gint    width;
	gint    height;

	pixels =
	    gtk_xine_get_current_frame_rgb (GTK_XINE (movieview->xine),
					    &width, &height);

	if (pixels)
	{
	    GdkPixbuf *pixbuf;

	    pixbuf =
		gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB, FALSE,
					  8, width, height, 3 * width,
					  free_rgb_buffer, NULL);

	    if (pixbuf)
	    {
		ZAlbumCell *cell;
		gint    scale_width;
		gint    scale_height;
		GdkPixbuf *scale_pixbuf;
		GdkPixmap *pixmap;
		GdkBitmap *mask;
		gchar  *path;
		gchar  *cache_dir;
		mode_t  mode = 0755;
		gint    pixbuf_width;
		gint    pixbuf_height;

		pixbuf_width = gdk_pixbuf_get_width (pixbuf);
		pixbuf_height = gdk_pixbuf_get_height (pixbuf);

		if (pixbuf_width > 100 || pixbuf_height > 100)
		{
		    if (((float) 100 / pixbuf_width)
			< ((float) 100 / pixbuf_height))
		    {
			scale_width = 100;
			scale_height =
			    (float) scale_width / pixbuf_width *
			    pixbuf_height;
			if (scale_height < 1)
			    scale_height = 1;
		    }
		    else
		    {
			scale_height = 100;
			scale_width =
			    (float) scale_height / pixbuf_height *
			    pixbuf_width;
			if (scale_width < 1)
			    scale_width = 1;
		    }
		}
		else
		{
		    scale_width = pixbuf_width;
		    scale_height = pixbuf_height;
		}

		scale_pixbuf =
		    gdk_pixbuf_scale_simple (pixbuf, scale_width,
					     scale_height,
					     conf.thumbnail_quality);

		gdk_pixbuf_unref (pixbuf);

		cell =
		    ZLIST_CELL_FROM_INDEX (ZLIST (thumbview->album),
					   ZLIST (thumbview->album)->focus);

		path =
		    g_strconcat (browser->current_path->str, "/", cell->name,
				 PORNVIEW_CACHE_THUMB_EXT, NULL);

		cache_dir = cache_get_location (path, FALSE, NULL, &mode);

		if (cache_ensure_dir_exists (cache_dir, mode))
		{
		    gchar  *cache_path;

		    cache_path =
			g_strconcat (cache_dir, "/", cell->name,
				     PORNVIEW_CACHE_THUMB_EXT, NULL);
		    if (pixbuf_to_file_as_png (scale_pixbuf, cache_path))
		    {
			struct utimbuf ut;

			ut.actime = ut.modtime = filetime (path);

			if (ut.modtime > 0)
			    utime (cache_path, &ut);
		    }

		    g_free (cache_path);
		}

		gdk_pixbuf_render_pixmap_and_mask (scale_pixbuf, &pixmap,
						   &mask, 128);
		zalbum_set_pixmap (ZALBUM (thumbview->album),
				   ZLIST (thumbview->album)->focus, pixmap,
				   mask);

		if (pixmap)
		    gdk_pixmap_unref (pixmap);

		if (mask)
		    gdk_bitmap_unref (mask);

		gdk_pixbuf_unref (scale_pixbuf);
		
		g_free (path);
		g_free (cache_dir);
	    }
	}
    }
}

static void
cb_movieview_play (GtkWidget * widget, MovieView * mv)
{
    ZAlbumCell *cell;

    if (gtk_xine_get_speed (GTK_XINE (mv->xine)) != SPEED_NORMAL)
	gtk_xine_set_speed (GTK_XINE (mv->xine), SPEED_NORMAL);
    else
    {
	cell =
	    ZLIST_CELL_FROM_INDEX (ZLIST (thumbview->album),
				   ZLIST (thumbview->album)->focus);

	if (cell)
	{
	    if (!gtk_xine_is_playing (GTK_XINE (mv->xine)))
	    {
		gtk_xine_play (GTK_XINE (movieview->xine), cell->name, 0, 0);
		gtk_adjustment_set_value (GTK_ADJUSTMENT (mv->seek_adj), 0);
	    }
	}
    }
}

static void
cb_movieview_pause (GtkWidget * widget, MovieView * mv)
{
    if (gtk_xine_is_playing (GTK_XINE (mv->xine)))
	gtk_xine_set_speed (GTK_XINE (mv->xine), SPEED_PAUSE);
}

static void
cb_movieview_stop (GtkWidget * widget, MovieView * mv)
{
    if (gtk_xine_is_playing (GTK_XINE (mv->xine)))
	gtk_xine_stop (GTK_XINE (mv->xine));
}

static void
cb_movieview_forward (GtkWidget * widget, MovieView * mv)
{
    if (gtk_xine_is_playing (GTK_XINE (mv->xine)))
	gtk_xine_set_speed (GTK_XINE (mv->xine), SPEED_FAST_4);

}

static void
cb_movieview_prev (GtkWidget * widget, MovieView * mv)
{
    thumbview_select_prev ();
}

static void
cb_movieview_next (GtkWidget * widget, MovieView * mv)
{
    thumbview_select_next ();
}

static  gint
cb_movieview_key_press (GtkWidget * widget, GdkEventKey * event,
			MovieView * mv)
{
    if (mv->is_hide)
	return FALSE;

    switch (event->keyval)
    {
      case GDK_Return:
	  movieview_fullscreen_mode_set ();
	  return TRUE;

      case GDK_t:
      case GDK_T:
	  movieview_create_thumb ();
	  return TRUE;

      case GDK_Q:
      case GDK_q:
      case GDK_Escape:
	  if (mv->is_fullscreen)
	  {
	      mv->is_fullscreen = FALSE;

	      gtk_xine_set_fullscreen (GTK_XINE (mv->xine), FALSE);

	      gdk_keyboard_ungrab (GDK_CURRENT_TIME);
	      gtk_grab_remove (movieview->event_box);

	      gtk_widget_grab_focus (GTK_WIDGET (movieview->event_box));
	  }
	  return TRUE;
    }

    return FALSE;
}

static  gint
cb_movieview_button_press (GtkWidget * widget, GdkEventButton * event,
			   MovieView * mv)
{
    if (mv->is_hide)
	return FALSE;

    gtk_widget_grab_focus (GTK_WIDGET (mv->event_box));

    return FALSE;
}


static  gboolean
cb_seek_pressed (GtkWidget * widget, GdkEventButton * event, MovieView * mv)
{
    if (mv->is_hide)
	return FALSE;

    g_return_val_if_fail (mv, FALSE);

    mv->seek_lock = TRUE;

    return FALSE;
}


static  gboolean
cb_seek_released (GtkWidget * widget, GdkEventButton * event, MovieView * mv)
{
    if (mv->is_hide)
	return FALSE;

    g_return_val_if_fail (mv, FALSE);


    if (gtk_xine_is_playing (GTK_XINE (mv->xine)))
    {
	ZAlbumCell *cell;

	cell =
	    ZLIST_CELL_FROM_INDEX (ZLIST (thumbview->album),
				   ZLIST (thumbview->album)->focus);

	gtk_xine_play (GTK_XINE (movieview->xine), cell->name,
		       (gint) GTK_ADJUSTMENT (mv->seek_adj)->value, 0);
    }

    mv->seek_lock = FALSE;

    return FALSE;
}

static int
cb_update_seek (gpointer data)
{
    MovieView *mv = data;

    if (mv->is_hide)
	return TRUE;

    if (mv->seek_lock == FALSE)
    {
	if (gtk_xine_is_playing (GTK_XINE (mv->xine)))
	{
	    gfloat  pos = 0.0;

	    pos = (gfloat) gtk_xine_get_position (GTK_XINE (mv->xine));


	    gtk_adjustment_set_value (GTK_ADJUSTMENT (mv->seek_adj), pos);
	}
	else
	    gtk_adjustment_set_value (GTK_ADJUSTMENT (movieview->seek_adj),
				      0);
    }

    return TRUE;
}

static void
free_rgb_buffer (guchar * pixels, gpointer data)
{
    g_free (pixels);
}

#endif /* ENABLE_XINE */
