/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

gboolean
pixbuf_to_file_as_png (GdkPixbuf * pixbuf, gchar * filename)
{
    FILE   *handle;
    char   *buffer;
    gboolean has_alpha;
    int     width, height, depth, rowstride;
    guchar *pixels;
    png_structp png_ptr;
    png_infop info_ptr;
    png_text text[2];
    int     i;

    g_return_val_if_fail (pixbuf != NULL, FALSE);
    g_return_val_if_fail (filename != NULL, FALSE);
    g_return_val_if_fail (filename[0] != '\0', FALSE);

    handle = fopen (filename, "wb");
    if (handle == NULL)
	return FALSE;

    png_ptr =
	png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (png_ptr == NULL)
    {
	fclose (handle);
	return FALSE;
    }

    info_ptr = png_create_info_struct (png_ptr);
    if (info_ptr == NULL)
    {
	png_destroy_write_struct (&png_ptr, (png_infopp) NULL);
	fclose (handle);
	return FALSE;
    }

    if (setjmp (png_ptr->jmpbuf))
    {
	png_destroy_write_struct (&png_ptr, &info_ptr);
	fclose (handle);
	return FALSE;
    }

    png_init_io (png_ptr, handle);

    has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
    width = gdk_pixbuf_get_width (pixbuf);
    height = gdk_pixbuf_get_height (pixbuf);
    depth = gdk_pixbuf_get_bits_per_sample (pixbuf);
    pixels = gdk_pixbuf_get_pixels (pixbuf);
    rowstride = gdk_pixbuf_get_rowstride (pixbuf);

    png_set_IHDR (png_ptr, info_ptr, width, height,
		  depth, PNG_COLOR_TYPE_RGB_ALPHA,
		  PNG_INTERLACE_NONE,
		  PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);

    /*
     * Some text to go with the png image 
     */
    text[0].key = "Title";
    text[0].text = filename;
    text[0].compression = PNG_TEXT_COMPRESSION_NONE;
    text[1].key = "Software";
    text[1].text = "Pornview";
    text[1].compression = PNG_TEXT_COMPRESSION_NONE;
    png_set_text (png_ptr, info_ptr, text, 2);

    /*
     * Write header data 
     */
    png_write_info (png_ptr, info_ptr);

    /*
     * if there is no alpha in the data, allocate buffer to expand into 
     */
    if (has_alpha)
	buffer = NULL;
    else
	buffer = g_malloc (4 * width);

    /*
     * pump the raster data into libpng, one scan line at a time 
     */
    for (i = 0; i < height; i++)
    {
	if (has_alpha)
	{
	    png_bytep row_pointer = pixels;
	    png_write_row (png_ptr, row_pointer);
	}
	else
	{
	    /*
	     * expand RGB to RGBA using an opaque alpha value 
	     */
	    int     x;
	    char   *buffer_ptr = buffer;
	    char   *source_ptr = pixels;
	    for (x = 0; x < width; x++)
	    {
		*buffer_ptr++ = *source_ptr++;
		*buffer_ptr++ = *source_ptr++;
		*buffer_ptr++ = *source_ptr++;
		*buffer_ptr++ = 255;
	    }
	    png_write_row (png_ptr, (png_bytep) buffer);
	}
	pixels += rowstride;
    }

    png_write_end (png_ptr, info_ptr);
    png_destroy_write_struct (&png_ptr, &info_ptr);

    g_free (buffer);

    fclose (handle);
    return TRUE;
}

void
pixbuf_render_pixmap_and_mask (GdkPixbuf * pixbuf, GdkPixmap ** pixmap_return,
			       GdkBitmap ** mask_return, GdkInterpType dither)
{
    gint    width;
    gint    height;

    g_return_if_fail (pixbuf != NULL);

    width = gdk_pixbuf_get_width (pixbuf);
    height = gdk_pixbuf_get_height (pixbuf);

    if (pixmap_return)
    {
	GdkGC  *gc;

	*pixmap_return = gdk_pixmap_new (NULL, width, height,
					 gdk_rgb_get_visual ()->depth);
	gc = gdk_gc_new (*pixmap_return);
	gdk_pixbuf_render_to_drawable (pixbuf, *pixmap_return, gc,
				       0, 0, 0, 0,
				       width, height, dither, 0, 0);
	gdk_gc_unref (gc);
    }

    if (mask_return)
    {
	if (gdk_pixbuf_get_has_alpha (pixbuf))
	{
	    *mask_return = gdk_pixmap_new (NULL, width, height, 1);
	    gdk_pixbuf_render_threshold_alpha (pixbuf, *mask_return,
					       0, 0, 0, 0,
					       width, height, 127);
	}
	else

	    *mask_return = NULL;
    }
}

GtkWidget *
pixbuf_create_pixmap_from_xpm_data (const char **data)
{
    GdkPixbuf *pixbuf = NULL;
    GdkPixmap *pixmap = NULL;
    GdkBitmap *mask = NULL;
    GtkWidget *iconw = NULL;

    pixbuf = gdk_pixbuf_new_from_xpm_data (data);
    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, 127);
    gdk_pixbuf_unref (pixbuf);
    iconw = gtk_pixmap_new (pixmap, mask);

    return iconw;
}
